@@ -8,12 +8,38 @@
  * Copyright (C) 2002-2011
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: ifdhandler.h 6302 2012-05-18 11:04:40Z rousseau $
+Redistribution and use in source and binary forms, with or without
+modification, are permitted provided that the following conditions
+are met:
+
+1. Redistributions of source code must retain the above copyright
+   notice, this list of conditions and the following disclaimer.
+2. Redistributions in binary form must reproduce the above copyright
+   notice, this list of conditions and the following disclaimer in the
+   documentation and/or other materials provided with the distribution.
+3. The name of the author may not be used to endorse or promote products
+   derived from this software without specific prior written permission.
+
+Changes to this license can be made only by the copyright author with
+explicit written consent.
+
+THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
+IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
+IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
+INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
+NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
+THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * $Id: ifdhandler.h 6822 2014-01-13 15:21:25Z rousseau $
  */
 
 /**
  * @file
- * @defgroup IFDHandler
+ * @defgroup IFDHandler IFDHandler
  * @brief This provides reader specific low-level calls.
 
 The routines specified hereafter will allow you to write an IFD handler
@@ -414,16 +440,15 @@ port used by each reader.
 
 - libusb
 
-  For USB drivers using libusb http://libusb.sourceforge.net/ for USB
+  For USB drivers using libusb-1.0 http://libusb.sourceforge.net/ for USB
   abstraction the @p DeviceName the string may be generated by:
 
   @code
-  printf("usb:%04x/%04x:libusb:%s:%s",
-    idVendor, idProduct,
-    bus->dirname, dev->filename)
+  printf("usb:%04x/%04x:libusb-1.0:%d:%d:%d",
+    idVendor, idProduct, bus_number, device_address, interface)
   @endcode
 
-  So it is something like: <tt>usb:08e6/3437:libusb:001:042</tt> under
+  So it is something like: <tt>usb:08e6/3437:libusb-1.0:7:99:0</tt> under
   GNU/Linux.
 
 - libudev
